/*******************************************************************************************************
    Search for classical Capacity of Quantum Channels(SeCQC) Version 1.0, a program that can calculate the maximum accessible information 
    and the corresponding optimal statistical operators as well as POVMs given a certain quantum channel.

    Copyright (C) 2010 J.W. Shang, K.L. Lee and B.-G. Englert.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. 
    See the GNU General Public License for more details.

	You can refer to http://www.gnu.org/licenses/ for the GNU General Public License.
	
	Detail manual for the program could be downloaded from http://www.quantumlah.org/publications/software/SeCQC/.
		
    The code for Graphical User Interface (GUI) is following the book 'Cross-Platform GUI Programming with WxWidgets'.
********************************************************************************************************/
//******************************************************************************************************/
//	Compiled under Windows XP Service Pack 3, with wxWidgets 2.8.10, MinGW 5.1.6, MSYS 1.0.11
//
//	Compile with the following command
//		g++ MI.cpp `wx-config --libs` `wx-config --cxxflags` -o SeCQC
//	in MSYS shell.
//	
//  If you face problems running the program in a Linux environment, try
//      export LD LIBRARY PATH=/usr/local/lib
//  in command window.
//
//******************************************************************************************************/

// GUI header file for program to calculate the  maximum mutual information
//-----------------------------------------------
// Class definitions
//-----------------------------------------------
class myApp:public wxApp
{
	public:
		virtual bool OnInit();
};

class matrixGrid:public wxGrid
{
	public:
		matrixGrid(wxPanel *panel, int wxID);
		
		void setDimension(int x);
		void changeElement(int i, int j, double f);
		void setGrid(vector< vector<dcomp> > a, int m, int n, int ri);
};

class myPanel:public wxPanel
{
	public:    	
    	myPanel(wxFrame *frame, int x, int y);

    	wxPanel *my_panel;
    	
     wxNotebook *m_book;
    	
       wxButton *m_calcButton,
    	        *m_directChange;
    	
     wxRadioBox *m_CapRadio,
                *m_rhoRadio,
                *m_povmRadio;
    	
     wxTextCtrl *m_displayMI,
    	        *m_tol_MI,
       	        *m_output,
    	        *m_directory;
    	
   wxStaticText *m_CapDimensionText,
                *m_rhoDimensionText;
		
  	 wxSpinCtrl *m_JTEXT,
      		    *m_KTEXT,
    		    *m_CapDimension,
    		    *m_rhoDimension,
    		    *m_MTEXT,
    		    *m_CTEXT,
    		    *m_directGradChance,
    		    *m_InputCapC,
    		    *m_InputCapM,
		        *m_Displayrho,
		        *m_DisplayPOVM;
				   
	 matrixGrid *CapGrid,
	            *rhoGrid,
 		        *povmGrid;
    	
	  	 MutInf *m_information;				   
		
		//event handlers to manipulate CapGrid		   
		void ChangeCapNumber(wxSpinEvent&);
		void ChangeCapNumberFunction();
		void ChangeCapDimension(wxSpinEvent&);
		void ChangeCapDimensionFunction();
		void OnCapCellValueChanged(wxGridEvent&);
		void UpdateCap(wxSpinEvent&);
		
		//event handlers to manipulate rhoGrid and povmGrid
		void ChangerhoNumber(wxSpinEvent&);
		void ChangePOVMNumber(wxSpinEvent&);
		void ChangerhoNumberFunction();
		void ChangePOVMNumberFunction();
		void ChangerhoDimension(wxSpinEvent&);
		void ChangePOVMDimension(wxSpinEvent&);
		void ChangerhoDimensionFunction();
		void ChangePOVMDimensionFunction();
		void Updaterho(wxSpinEvent&);
		void UpdatePOVM(wxSpinEvent&);
		void OnrhoCellValueChanged(wxGridEvent&);
		void OnPOVMCellValueChanged(wxGridEvent&);
		
		void OnCalc(wxCommandEvent&);
		void OnReset(wxCommandEvent&);
		void OnSave(wxCommandEvent&);
	private:
    	DECLARE_EVENT_TABLE()
};

class myFrame:public wxFrame
{
	public:
		myFrame(const wxString&, const wxPoint&, const wxSize&);
		myPanel *m_panel;
	
		int extractParameter(wxString&);
		void extractCapElement(wxString&, int, int, int&);
		void OnImport(wxCommandEvent&);
		void OnQuit(wxCommandEvent&);
		void OnAbout(wxCommandEvent&);
	private:
		DECLARE_EVENT_TABLE()
};
//----------------------------------------------------------
//	(non-standard) event identifier defined
//----------------------------------------------------------
enum
{
    wx_Calc = 1,

    BOOKCTRL = wxID_HIGHEST + 1,
    BUTTON_Calc = wxID_HIGHEST + 2,
    BUTTON_Reset = wxID_HIGHEST + 3,
    BUTTON_Save = wxID_HIGHEST + 4,
    wxID_JTEXT = wxID_HIGHEST + 5,
    wxID_KTEXT = wxID_HIGHEST + 6,
    wxID_CapDimension = wxID_HIGHEST + 7,
    wxID_CapDimensionText = wxID_HIGHEST + 8,
    wxID_rhoDimension = wxID_HIGHEST + 9,
    wxID_rhoDimensionText = wxID_HIGHEST + 10,
    wxID_MTEXT = wxID_HIGHEST + 11,
    wxID_CTEXT = wxID_HIGHEST + 12,
    wxID_InputCapC = wxID_HIGHEST + 13,
    wxID_InputCapM = wxID_HIGHEST + 14,
    wxID_Displayrho = wxID_HIGHEST + 15,
    wxID_DisplayPOVM = wxID_HIGHEST + 16,
    wxID_DisplayMI = wxID_HIGHEST + 17,
    wxID_CapGrid = wxID_HIGHEST + 18,
    wxID_rhoGrid = wxID_HIGHEST + 19,
    wxID_povmGrid = wxID_HIGHEST + 20,
    wxID_IMPORT = wxID_HIGHEST + 21,
};
